import functools

from .event_types import SlackEventType
from .views import SlackEvent


def slackevent(app_name_or_func):
    def _slackevent(func, app_name=None):
        @functools.wraps(func)
        def wrapped(*args, **kw):
            payload = kw['slack_payload']
            if (payload.get('event')
                    and payload['event']['type'] == func.__name__
                    and app_name == kw['derived_app_name']):
                return func(payload, *args, **kw)

        SlackEvent.connect(wrapped)
        return wrapped

    if callable(app_name_or_func):
        return _slackevent(app_name_or_func)

    return functools.partial(_slackevent, app_name=app_name_or_func)


def slackaction(app_name_or_func):
    def _slackaction(func, app_name=None):
        @functools.wraps(func)
        def wrapped(*args, **kw):
            payload = kw['slack_payload']
            if not app_name == kw['derived_app_name']:
                return

            action_or_cb_id = payload.get('action_id',
                                payload.get('callback_id',
                                  payload.get('view', {}).get('callback_id')
                                )
                              )
            default = [{'action_id': action_or_cb_id}]
            for action in default + payload.get('actions', []):
                if func.__name__ == action['action_id']:
                    return func(payload, *args, **kw)

        SlackEvent.connect(wrapped)
        return wrapped

    if callable(app_name_or_func):
        return _slackaction(app_name_or_func)

    return functools.partial(_slackaction, app_name=app_name_or_func)
